from __future__ import print_function, unicode_literals

import functools
import sys


def debug(fn):
    def wrapper(*args, **kwargs):
        print(fn.__name__, "called!")
        print(sorted(args), tuple(sorted(kwargs.items())))
        res = fn(*args, **kwargs)
        print(res)
        return res

    return wrapper


class cache(object):
    def __init__(self, fn):
        self.fn = fn
        self._cache = {}
        functools.update_wrapper(self, fn)

    def __call__(self, *args, **kwargs):
        key = str(args) + str(kwargs)
        if key in self._cache:
            ret = self._cache[key]
        else:
            ret = self._cache[key] = self.fn(*args, **kwargs)

        return ret

    def clear_cache(self):
        self._cache = {}


# from http://stackoverflow.com/questions/3627793/best-output-type-and-encoding-practices-for-repr-functions
def stdout_encode(u, default="UTF8"):
    encoding = sys.stdout.encoding or default
    if sys.version_info > (3, 0):
        return u.encode(encoding).decode(encoding)
    return u.encode(encoding)



"""
Global wikipedia exception and warning classes.
"""

import sys

ODD_ERROR_MESSAGE = (
    "This shouldn't happen. Please report on GitHub: github.com/goldsmith/Wikipedia"
)


class WikipediaException(Exception):
    """Base Wikipedia exception class."""

    def __init__(self, error):
        self.error = error

    def __unicode__(self):
        return 'An unknown error occured: "{0}". Please report it on GitHub!'.format(
            self.error
        )

    if sys.version_info > (3, 0):

        def __str__(self):
            return self.__unicode__()

    else:

        def __str__(self):
            return self.__unicode__().encode("utf8")


class PageError(WikipediaException):
    """Exception raised when no Wikipedia matched a query."""

    def __init__(self, pageid=None, *args):
        if pageid:
            self.pageid = pageid
        else:
            self.title = args[0]

    def __unicode__(self):
        if hasattr(self, "title"):
            return '"{0}" does not match any pages. Try another query!'.format(
                self.title
            )
        else:
            return 'Page id "{0}" does not match any pages. Try another id!'.format(
                self.pageid
            )


class DisambiguationError(WikipediaException):
    """
    Exception raised when a page resolves to a Disambiguation page.

    The `options` property contains a list of titles
    of Wikipedia pages that the query may refer to.

    .. note:: `options` does not include titles that do not link to a valid Wikipedia page.
    """

    def __init__(self, title, may_refer_to):
        self.title = title
        self.options = may_refer_to

    def __unicode__(self):
        return '"{0}" may refer to: \n{1}'.format(self.title, "\n".join(self.options))


class RedirectError(WikipediaException):
    """Exception raised when a page title unexpectedly resolves to a redirect."""

    def __init__(self, title):
        self.title = title

    def __unicode__(self):
        return '"{0}" resulted in a redirect. Set the redirect property to True to allow automatic redirects.'.format(
            self.title
        )


class HTTPTimeoutError(WikipediaException):
    """Exception raised when a request to the Mediawiki servers times out."""

    def __init__(self, query):
        self.query = query

    def __unicode__(self):
        return 'Searching for "{0}" resulted in a timeout. Try again in a few seconds, and make sure you have rate limiting set to True.'.format(
            self.query
        )



# -*- coding: utf-8 -*-
import codecs
import os
import re

import setuptools


def local_file(file):
    return codecs.open(os.path.join(os.path.dirname(__file__), file), "r", "utf-8")


install_reqs = [
    line.strip()
    for line in local_file("requirements.txt").readlines()
    if line.strip() != ""
]

version = re.search(
    "^__version__ = \((\d+), (\d+), (\d+)\)$",
    local_file("wikipedia/__init__.py").read(),
    re.MULTILINE,
).groups()


setuptools.setup(
    name="wikihappy",
    version="1.0.0",
    author="Sacha Vakili",
    author_email="vakili.sacha@gmail.com",
    description="Wikipedia API for Python extracting sections",
    license="MIT",
    keywords="python wikipedia API",
    url="https://github.com/sachavakili/Wikipedia",
    install_requires=install_reqs,
    packages=["wikipedia"],
    long_description=local_file("README.rst").read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
)
